import pygame
from pygame.locals import *
import random

from pgu import timer

rr = random.randrange

class Part:
    def __init__(self,x,y):
        x,y = float(x),float(y)
        self.x,self.y = x,y
        self._x,self._y = x,y
        self.min = 4.0
        self.max = 16.0
        self.radius = 16.0
        self.center = 2.0
        self.search = 64.0
        
def flock(parts):
    #calculate velocities
    for p in parts:
        p.vx,p.vy = p.x-p._x,p.y-p._y
        p._x,p._y = p.x,p.y
    
    
    #calculate near particles
    for p in parts:
        if not hasattr(p,'_flock_frame'): p._flock_frame = 0
        if p._flock_frame%2 == 0:
            step = p.search
            p.near = []
            p.ax,p.ay = 0,0
            p.avx,p.avy = 0,0
            for _p in parts:
                dx,dy = _p.x-p.x,_p.y-p.y
                dist = (dx*dx+dy*dy)**0.5
                if dist < step: #48 = 6 neighbors, 64 = 12 neighbors.
                    p.near.append(_p)
                    p.ax += _p.x
                    p.ay += _p.y
                    p.avx += _p.vx
                    p.avy += _p.vy
            total = len(p.near)
            p.ax /= total
            p.ay /= total
            p.avx /= total
            p.avy /= total
            
    #display average total
#     total = 0
#     for p in parts: total += len(p.near)
#     print 'neighbors', total/len(parts)
#             
    #"""
    #add average velocity to all
    for p in parts:
        #for _p in p.near:
        avx,avy = p.avx,p.avy
        vx,vy = p.vx,p.vy
        #p.x,p.y = p.x + (p.vx+avx)/2, p.y + (p.vy+avy)/2
        p.x,p.y = p.x+avx,p.y+avy
        #p.x,p.y = p.x+(vx+avx)/2,p.y+(vy+avy)/2
    #"""
    
    #"""
    #move towards the center
    for p in parts:
        step = p.center
        ax,ay = p.ax,p.ay
        dx,dy = ax-p.x,ay-p.y
        #dist = abs(dx)+abs(dy)
        dist = (dx*dx+dy*dy)**0.5
        if dist > step:
            p.x,p.y = p.x+step*dx/dist,p.y+step*dy/dist
            
    #make sure we're going at a decent min speed
    for p in parts:
        step = p.min
        dx,dy = p.x-p._x,p.y-p._y
        dist = (dx*dx+dy*dy)**0.5
        while dist < 0.5:
            dx,dy = random.randrange(-1,2),random.randrange(-1,2)
            dist = (dx*dx+dy*dy)**0.5
        if dist < step:
            p.x,p.y = p._x+dx*step/dist,p._y+dy*step/dist

    
    #keep away from neighbords
    for p in parts:
        for _p in p.near:
            if p != _p:
                dx,dy = _p.x-p.x,_p.y-p.y
                dist = (dx*dx+dy*dy)**0.5
                r = p.radius+_p.radius
                if dist < r:
                    if dist == 0: dist,dx = 1,1
                    #inc = step-dist
                    #inc /= 2
                    inc = r/2.0
                    
                    mx,my = (p.x+_p.x)/2,(p.y+_p.y)/2
                    
                    p.x,p.y = mx - (inc*dx/dist), my - (inc*dy/dist)
                    _p.x,_p.y = mx + (inc*dx/dist), my + (inc*dy/dist)
                    
                    #p._x,p._y = p._x + inc*dx/dist,p._y+inc*dy/dist
                    
                    #_p._x,_p._y = _p._x - inc*dx/dist,_p._y-inc*dy/dist
                    
    #make sure movement is limited to a reasonable amout
    for p in parts:
        step = p.max
        dx,dy = p.x-p._x,p.y-p._y
        dist = (dx*dx+dy*dy)**0.5
        if dist > step:
            p.x,p.y = p._x+dx*step/dist,p._y+dy*step/dist
            
                
            
if __name__ == '__main__':
    try:
        #0/0 #HACK: so i can CTRL-c out
        import psyco
        psyco.profile()
        print 'psyco installed'
    except:
        print 'psyco not installed'

    SW,SH = 640,480
    screen = pygame.display.set_mode((SW,SH))
    
    parts = [Part(rr(0,SW),rr(0,SH)) for n in xrange(0,64)]
    
    #t = timer.Timer(40)
    t = timer.Speedometer()
    pygame.time.get_ticks()
    
    _quit = False
    while not _quit:
        for e in pygame.event.get():
            if e.type is QUIT: _quit = True
            if e.type is KEYDOWN and e.key == K_ESCAPE: _quit = True
        
        screen.fill((0,0,0))
        p = parts[0]
        p.x,p.y = pygame.mouse.get_pos()
        p.radius = 64
        pygame.draw.circle(screen,(0,0,128),(int(p.x),int(p.y)),int(p.radius))
        flock(parts)
        for p in parts:
            p.x = max(0,min(SW,p.x))
            p.y = max(0,min(SH,p.y))
            screen.fill((255,255,255),pygame.Rect(int(p.x),int(p.y),4,4))
        pygame.display.flip()
        
        r = t.tick()
        if r: print r
        
